

function x=linsolveUT( A, b)
	b=b';
	n=length(b);
	x=zeros(1,n);
	x(n)=b(n)/A(n,n);
	for k=n-1:-1:1
		x(k) = (b(k)-sum(A(k,k+1:n).*x(k+1:n)))/A(k,k);
	end
	x=x';
end

function x=linsolveLT( A, b)
	b=b';
	n=length(b);
	x=zeros(1,n);
	x(1)=b(1)/A(1,1);
	for k=2:n
		x(k) = (b(k)-sum(A(k,1:k-1).*x(1:k-1)))/A(k,k);
	end
	x=x';
end



function coef = splinefit(x,y,delta,S) 
%% usage:
%% Cubic spline fit for data points x/y (C. H. Reinsch). coef is a piecewise polynomial, 
%% see function ppval() for details of the format. delta is a vector of tolerances of y.
%% Larger S yield smoother curves with larger deviation.
%% $X{
%% >> x=0:5;y=rand(1,6);
%% >> d=ones(1,6)*0.02;
%% >> plot(x,y,'r*');hold; 
%% >> xp=0:0.1:5;
%% >> for S=[1,50,500]
%% >    p=splinefit(x,y,d,S);  
%% >    plot(xp,ppval(p,xp))
%% >  end}
%% $I{splinefit.png}
%% $L{spline,ppval}

	n = length(x);
	epsilon = 0.0000000001; 

	h = x(2:n)  - x(1:n-1); 
	D = diag(delta); 
	tOben = (1/3)*h(2:n-2); 
	tMitte = (2/3)*(h(1:n-2) + h(2:n-1)); 
	tUnten = (1/3)*h(2:n-2); 
	T = diag(tOben, 1) + diag(tMitte) + diag(tUnten,-1); 
	qOben = 1 ./ h(1:n-2); 
	qMitte =  -1 ./ h(1:n-2) - 1 ./ h(2:n-1); 
	qUnten = 1 ./ h(2:n-1);  % Fehler!
	Q = zeros(n,n-2);
	Q(1:n-2,1:n-2)=diag(qOben);
	Q(2:n-1,1:n-2)=Q(2:n-1,1:n-2)+diag(qMitte);
	Q(3:n,1:n-2)=Q(3:n,1:n-2)+diag(qUnten); % Fehler

	q = 0; 
	F2 = S + 1; 
	while (F2 > S + epsilon)
		Rq=Q'*D*D*Q + q*T;
		R = chol(sparse(Rq));
		Ru = linsolveLT(R', Q'*y'); 
		u = linsolveUT(R, Ru); 
		F = D*Q*u; 
		F2 = F'*F; 
		if (F2 > S) 
			f = T*u; 
			v = linsolveLT(R',f); 
			q = q - (F2 - sqrt(S*F2))/(q*v'*v - u'*f);
		end 
	end 

	c =zeros(n,1); c(2:n-1,1)=q*u;
	a = y'  - D*F; 
	d = (c(2:n,1)  - c(1:n-1,1))./(3*h)'; 
	b = (a(2:n,1)  - a(1:n-1,1))./h(1:n-1)'  - c(1:n-1,1).*h(1:n-1)'-d(1:n-1,1).*h(1:n-1)'.*h(1:n-1)'; 
	c = c(1:n-1,1); 
	a = a(1:n-1,1); 
	coef = [d,c,b,a,x(1:n-1)'];
end

